<div id="login">
    <div class="login-form" data-base-target="layout">
        <div role="status" class="sr-only">
            <?= $this->translate(
                'Welcome to Icinga Web 2. For users of the screen reader Jaws full and expectant compliant'
                . ' accessibility is possible only with use of the Firefox browser. VoiceOver on Mac OS X is tested on'
                . ' Chrome, Safari and Firefox.'
            ) ?>
        </div>
        <div class="logo-wrapper"><div id="icinga-logo" aria-hidden="true"></div></div>
        <?php if ($requiresSetup): ?>
            <p class="config-note"><?= sprintf(
                $this->translate(
                    'It appears that you did not configure Icinga Web 2 yet so it\'s not possible to log in without any defined '
                    . 'authentication method. Please define a authentication method by following the instructions in the'
                    . ' %1$sdocumentation%3$s or by using our %2$sweb-based setup-wizard%3$s.'
                ),
                '<a href="https://icinga.com/docs/icinga-web-2/latest/doc/05-Authentication/#authentication" title="'
                    . $this->translate('Icinga Web 2 Documentation') . '">',
                '<a href="' . $this->href('setup') . '" title="' . $this->translate('Icinga Web 2 Setup-Wizard') . '">',
                '</a>'
            ) ?></p>
        <?php endif ?>
        <?= $this->form ?>
        <?= implode('', $this->loginButtons) ?>
        <div id="login-footer">
            <p>Icinga Web 2 &copy; 2013-<?= date('Y') ?></p>
            <?= $this->qlink($this->translate('icinga.com'), 'https://icinga.com') ?>
        </div>
    </div>
    <ul id="social">
        <li>
            <?= $this->qlink(
                null,
                'https://www.facebook.com/icinga',
                null,
                array(
                    'target'  => '_blank',
                    'icon'     => 'facebook-squared',
                    'title'    => $this->translate('Icinga on Facebook')
                )
            ) ?>
        </li>
        <li><?= $this->qlink(
                null,
                'https://github.com/Icinga',
                null,
                array(
                    'target'    => '_blank',
                    'icon'      => 'github-circled',
                    'title'     => $this->translate('Icinga on GitHub')
                )
            ) ?>
        </li>
    </ul>
</div>
<div id="orb-analytics" class="orb" ><?= $this->img('img/orb-analytics.png'); ?></div>
<div id="orb-automation" class="orb"><?= $this->img('img/orb-automation.png'); ?></div>
<div id="orb-cloud" class="orb"><?= $this->img('img/orb-cloud.png'); ?></div>
<div id="orb-icinga" class="orb"><?= $this->img('img/orb-icinga.png'); ?></div>
<div id="orb-infrastructure"  class="orb"><?= $this->img('img/orb-infrastructure.png'); ?></div>
<div id="orb-metrics" class="orb" ><?= $this->img('img/orb-metrics.png'); ?></div>
<div id="orb-notifactions" class="orb"><?= $this->img('img/orb-notifications.png'); ?></div>
