<div class="controls">
<?= $this->tabs ?>
<h1>Icinga Web 2 Icons</h1>
</div>

<div class="content">
<?php foreach ($this->font->glyphs as $icon): ?>
<div class="icon <?=
    $this->font->css_prefix_text . $icon->css
?>">
<?= $this->escape($icon->css) ?> <span class="icon-code">(0x<?= dechex($icon->code) ?>)</span>
</div>
<?php endforeach ?>
</div>
