<div class="controls">
    <?= /** @var \Icinga\Web\Widget\Tabs $tabs */ $this->tabs ?>
</div>
<div class="content chapter">
    <?= /** @var \Icinga\Module\Doc\Renderer\DocSectionRenderer $section */ $section ?>
</div>
