// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

// General styles

code {
  color: @icinga-blue;
  font-family: @font-family-fixed;
}

pre > code {
  color: inherit;
}

.chapter a {
  border-bottom: 1px dotted @gray-light;
  font-weight: @font-weight-bold;

  &:hover {
    border-bottom: 1px solid @text-color;
    text-decoration: none;
  }
}

.content {
  font-size: 1.167em;
}

.search-highlight {
  .rounded-corners();

  background: @icinga-blue;
  color: @text-color-on-icinga-blue;
  padding: 0 0.3em 0 0.3em;
}

.toc {
  counter-reset: li;
  list-style-type: none;
  margin: 0;
  padding: 0;

  li {
    counter-increment: li;
    margin-top: 0.25em;

    > .toc {
      margin-left: 2em;
    }

    a {
      &:before {
        color: @icinga-blue;
        content: counters(li,".") " ";
        display: inline-block;
        font-size: small;
        font-weight: @font-weight-bold;
        min-width: 1.5em;
        padding: 0.25em;
        text-align: center;
      }

      display: block;
    }
  }
}

// Table styles

table {
  margin-bottom: 1em;
  width: 100%;
}

tbody > tr:nth-child(odd) {
  background: @gray-light;
}

tbody > tr:nth-child(even) {
  background: @body-bg-color;
}

td, th {
  padding: 0.5em;
}

td {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

th {
  border-bottom: 2px solid @icinga-blue;
  font-weight: @font-weight-bold;
  text-align: left;
  text-transform: uppercase;
}

.icon {
  width: 33%;
  font-size: 1.5em;
  height: 2em;
  float: left;
}

.icon-code {
  font-size: 0.6em;
}
