<div class="controls">
    <?= $tabs ?>
</div>
<div class="content">
    <?= $this->qlink(
        $this->translate('Create a New Resource') ,
        'config/createresource',
        null,
        array(
            'class'             => 'button-link',
            'data-base-target'  => '_next',
            'icon'              => 'plus',
            'title'             => $this->translate('Create a new resource')
        )
    ) ?>
    <table class="table-row-selectable common-table" data-base-target="_next">
    <thead>
    <tr>
        <th><?= $this->translate('Resource') ?></th>
        <th></th>
    </tr>
    </thead>
    <tbody>
<?php foreach ($this->resources as $name => $value): ?>
    <tr>
        <td>
            <?php
            switch ($value->type) {
                case 'db':
                    $icon = 'database';
                    break;
                case 'ldap':
                    $icon = 'sitemap';
                    break;
                case 'ssh':
                    $icon = 'user';
                    break;
                case 'file':
                case 'ini':
                    $icon = 'doc-text';
                    break;
                default:
                    $icon = 'edit';
                    break;
            }
            ?>
            <?= $this->qlink(
                $name,
                'config/editresource',
                array('resource' => $name),
                array(
                    'icon'  => $icon,
                    'title' => sprintf($this->translate('Edit resource %s'), $name)
                )
            ) ?>
        </td>
        <td class="icon-col text-right">
            <?= $this->qlink(
                '',
                'config/removeresource',
                array('resource' => $name),
                array(
                    'class'   => 'action-link',
                    'icon'    => 'cancel',
                    'title'   => sprintf($this->translate('Remove resource %s'), $name)
                )
            ) ?>
        </td>
    </tr>
<?php endforeach ?>
    </tbody>
    </table>
</div>
