<?php

// SPDX-FileCopyrightText: 2019 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Web\View\Helper;

class IcingaCheckbox extends \Zend_View_Helper_FormCheckbox
{
    public function icingaCheckbox($name, $value = null, $attribs = null, ?array $checkedOptions = null)
    {
        if (! isset($attribs['id'])) {
            $attribs['id'] = $this->view->protectId('icingaCheckbox_' . $name);
        }

        $attribs['class'] = (isset($attribs['class']) ? $attribs['class'] . ' ' : '') . 'sr-only';
        $html = parent::formCheckbox($name, $value, $attribs, $checkedOptions);

        $class = 'toggle-switch';
        if (isset($attribs['disabled'])) {
            $class .= ' disabled';
        }

        return $html
            . '<label for="'
            . $attribs['id']
            . '" aria-hidden="true"'
            . ' class="'
            . $class
            . '"><span class="toggle-slider"></span></label>';
    }
}
